from django.urls import path

from .views import video_list, video_gallery_list, video_detail, video_gallery_detail

urlpatterns = [
    path('', video_list, name="video_list"),
    path('galleries/', video_gallery_list, name="video_gallery_list"),
    path('<slug:slug>/', video_detail, name='video_detail'),
    path('gallery/<slug:slug>/', video_gallery_detail, name='video_gallery_detail'),
]



import datetime
import time

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.template import Context
from django.template.loader import get_template


UserModel = get_user_model()
today = datetime.date.today()
one_day_ago = today - datetime.timedelta(days=1)
last_seen_timestamp = time.mktime(one_day_ago.timetuple())


class Command(BaseCommand):
    help = 'Sends email digest of new activity to users who requested one.'

    def handle(self, *args, **options):
        count = 0
        site = Site.objects.get_current()
        for user in UserModel.objects.filter(get_digest=1):
            subject = '{} Digest for {}'.format(site.name, today.strftime("%A, %B %e"))

            watchlist = []
            watch_objects = [w.content_object for w in user.watch_set.all()]
            for obj in watch_objects:
                if hasattr(obj, 'modified') and (obj.modified.date() > one_day_ago or obj.modified_int > last_seen_timestamp):
                    watchlist.append(obj)

            c = Context({
                'user': user,
                'subject': subject,
                'last_seen': one_day_ago,
                'today': today,
                'watchlist': watchlist,
                'site': site,
                'site_root': 'http://{}'.format(site.domain),
                'theme': user.theme
            })
            tmpl = get_template('users/digest.html')

            body = tmpl.render(c)
            msg = EmailMultiAlternatives(
                subject,
                body,
                settings.ADMINS[0]
                [user.email]
            )
            msg.attach_alternative(body, "text/html")
            msg.send()
            count += 1



import datetime

from django import template
from django.contrib.auth import get_user_model

UserModel = get_user_model()

register = template.Library()

today = datetime.date.today()
one_day_ago = datetime.datetime.now() - datetime.timedelta(days=1)


@register.assignment_tag
def get_birthday_list():
    """
    Returns today's birthdays
    Usage:
    {% get_birthday_list as birthday_list %}
    """
    return UserModel.objects.filter(is_active=True, birthday__month = today.month, birthday__day = today.day).values()


@register.assignment_tag
def get_new_members(count=5):
    """
    Returns most recent new members.
    Usage:
    {% get_new_members NUM as new_members %}
    with NUM being the number you want, or 5 by default

    or you can pass 'today' in, and get new members for today:
    {% get_new_members 'today' as new_members %}

    """
    if count != 'today':
        return UserModel.objects.filter(is_active=True).order_by('-id')[:count]
    return UserModel.objects.filter(is_active=True, date_joined__gte=one_day_ago).order_by('-id')



from django.urls import path

from .views import member_index, edit_profile, edit_settings, view_profile

urlpatterns = [
    path('', member_index, name="community_index"),
    path('edit-profile/', edit_profile, name="edit_profile"),
    path('edit-settings/', edit_settings, name="edit_settings"),
    path('<int:pk>/', view_profile, name="view_profile_by_id"),
    path('<slug:slug>/', view_profile, name='view_profile'),
]



import datetime

from django.contrib.auth import get_user_model
from django.template import Template, Context
from django.test import TestCase
from django.urls import reverse

UserModel = get_user_model()

today = datetime.date.today()


class TestUserProfileViews(TestCase):
    fixtures = ['users.json']

    def setUp(self):
        self.user = UserModel.objects.get(id=1)

    def test_index(self):
        """
        Test community index
        """
        response = self.client.get(reverse('community_index'))
        self.assertEqual(response.status_code, 200)
        self.assertTrue('object_list' in response.context)
        self.assertTemplateUsed("users/user_list.html")
        response = self.client.get(reverse('community_index') + '?state=ks')
        self.assertTrue('filter' in response.context)

    def test_detail(self):
        response = self.client.get(reverse('view_profile', args=[self.user.username]))
        self.assertEqual(response.status_code, 200)
        self.assertTrue('user' in response.context)
        response_user = response.context['user']
        self.assertEqual(response_user.id, self.user.id)
        self.assertEqual(response_user.username, self.user.username)
        self.assertEqual(response_user.get_display_name(), self.user.username)


    def test_birthdays_tag(self):
        """
        Test birthdays
        """
        out = Template(
            "{% load user_tags %}"
            "{% get_birthday_list as birthday_list %}"
            "{% for bday in birthday_list %}"
            "{{ user.username }},"
            "{% endfor %}"
        ).render(Context())
        todays_birthdays = ','.join(UserModel.objects.filter(birthday__month = today.month, birthday__day = today.day).values_list('username', flat=True))
        self.assertEqual(out, todays_birthdays)

    def test_new_members_tag(self):
        """
        Test new members list.
        Since there are no new members in the JSON, this should return nothing.
        """
        # test by count
        out = Template(
            "{% load user_tags %}"
            "{% get_new_members 5 as new_members %}"
            "{% for member in new_members %}"
            "{{ user.username }}"
            "{% endfor %}"
        ).render(Context())
        self.assertEqual(out, '')

        # test for today
        out = Template(
            "{% load user_tags %}"
            "{% get_new_members 'today' as new_members %}"
            "{% for member in new_members %}"
            "{{ user.username }}"
            "{% endfor %}"
        ).render(Context())
        self.assertEqual(out, '')



from django import template

from video.models import Video

register = template.Library()


@register.assignment_tag()
def get_video_list(count=5):
    """
    Returns recent videos limited to count.
    Usage: "{% get_video_list as video_list %}"
    """
    return Video.published.all()[:count]


@register.inclusion_tag('video/includes/show_video.html')
def show_video(video, top_video=False):
    """
    API-call-free, responsive friendly video embed. Don't forget to use it with the JS that tells Firefox not to use native video, and responsifying CSS.
    """
    return {
        'video': video,
        'top_video': top_video
    }



from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.db import models
from django.template.defaultfilters import slugify

from .helpers import get_youtube_data, get_vimeo_data, get_ustream_data
from .managers import VideoManager, PublishedVideoManager

from tango_shared.models import BaseContentModel


class Video(BaseContentModel):
    video_at_top = models.BooleanField(
        "Put video at top",
        default=False,
        help_text="""
            If checked, the video will appear preceding the body
            of any related content instead of following.
            """
    )
    url = models.CharField(
        max_length=200,
        blank=True,
        help_text=""""
            Acceptable sources are Youtube, Vimeo and Ustream.
            Please give the link URL, not the embed code.
            """
    )
    hide_info = models.BooleanField(
        "Hide title and description",
        default=False,
        help_text="""
            If checked, the video title and description will not be shown.
            Left unchecked, the video will attempt to display the title and description
            from the video source.
            """
    )

    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        blank=True,
        null=True
    )
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey()

    key = models.CharField(max_length=20, blank=True, editable=False)
    source = models.CharField(max_length=20, blank=True, editable=False)
    embed_src = models.CharField(max_length=200, blank=True, editable=False)
    thumb_url = models.CharField(max_length=200, blank=True, editable=False)

    # Managers
    objects = VideoManager()
    published = PublishedVideoManager()

    def __unicode__(self):
        return '{}: {}'.format(self.source, self.title)

    @models.permalink
    def get_absolute_url(self):
        return ('video_detail', [self.slug])

    def save(self, *args, **kwargs):
        if self.title and not self.slug:
            self.slug = slugify(self.title)
        if not self.embed_src:
            if 'youtube.com/watch' in self.url or 'youtu.be/' in self.url:
                self = get_youtube_data(self)

            if 'vimeo.com/' in self.url:
                self = get_vimeo_data(self)

            if 'ustream.tv/' in self.url:
                self = get_ustream_data(self)

        if self.key and self.embed_src:
            self.embed_src = """
                <iframe src="{}{}" height="360" width="100%%"></iframe>
                """.format(self.embed_src, self.key)
        super(Video, self).save()

    def get_image(self):
        return self.thumb_url


class VideoGallery(models.Model):
    title = models.CharField(max_length=200)
    slug = models.SlugField(
        max_length=200,
        help_text="Used for URLs. Will auto-fill, but can be edited with caution."
    )
    gallery_credit = models.CharField(max_length=200, blank=True)
    summary = models.TextField(blank=True)
    featured = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)
    published = models.BooleanField(default=True)
    video_collection = models.ManyToManyField(Video)

    class Meta:
        verbose_name_plural = "video galleries"

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ('video_gallery_detail', [self.slug])



from django.contrib import admin
from django.contrib.contenttypes.admin import GenericTabularInline

from .forms import VideoForm
from video.models import Video, VideoGallery


class VideoInline(GenericTabularInline):
    """
    Consistent inlined video for other content admin.
    """
    model = Video
    max_num = 2
    extra = 1
    fields = ('url', 'video_at_top', 'hide_info')


class VideoGalleryAdmin(admin.ModelAdmin):
    prepopulated_fields = {'slug': ('title',)}
    filter_horizontal = ['video_collection']


class VideoAdmin(admin.ModelAdmin):
    form = VideoForm

    prepopulated_fields = {'slug': ('title',)}
    list_display = ('title', 'source')
    fieldsets = (
        ('', {'fields': ('url', 'title', 'summary',)}),
        ('Admin fields', {
            'description': 'These should be filled in for you, but you can edit them.',
            'fields': ('slug', ),
            'classes': ['collapse']
        }),
    )


admin.site.register(Video, VideoAdmin)
admin.site.register(VideoGallery, VideoGalleryAdmin)



from django import forms
from django.contrib.auth.forms import UserCreationForm

from .models import Profile


class PublicProfileForm(forms.ModelForm):
    """
    Allows for profile and settings updates.
    Note that template checks for "Open board links" label.
    Anything below that will be in a 'settings' fieldset.
    """
    class Meta:
        model = Profile
        fields = (
            'display_name',
            'street_address',
            'city',
            'state',
            'country',
            'zipcode',
            'occupation',
            'interests',
            'birthday',
            'homepage',
            'bio',
            'avatar',
            'signature',
        )


class ProfileSettingsForm(forms.ModelForm):
    """
    Allows for modifying profile settings
    """
    class Meta:
        model = Profile
        fields = (
            'display_on_map',
            'show_signatures',
            'theme'
        )


class ProfileCreationForm(UserCreationForm):

    class Meta:
        model = Profile
        fields = ("username",)



import datetime
import json

from django.contrib.auth import get_user_model
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.http import HttpResponse
from django.shortcuts import render, get_object_or_404
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.views.generic import ListView
from django.views.generic.edit import UpdateView

from .filters import ProfileFilter
from .forms import PublicProfileForm, ProfileSettingsForm


UserModel = get_user_model()
past_year = datetime.datetime.now() - datetime.timedelta(days=365)


class MemberList(ListView):
    """
    Renders either default user list (paginated) or search results.
    To-do: split search to separate view, make pagination work better.
    """
    queryset = UserModel.objects.filter(is_active=1, last_login__gte=past_year).order_by('display_name').values()
    template_name = "users/user_list.html"
    paginate_by = 100

    def get_context_data(self, **kwargs):
        context = super(MemberList, self).get_context_data(**kwargs)
        if 'display_name' in self.request.GET or 'state' in self.request.GET:
            filter = ProfileFilter(self.request.GET, queryset=self.queryset)
        else:
            filter = ProfileFilter()
        context['filter'] = filter
        return context
member_index = MemberList.as_view()


class EditProfile(UpdateView):
    model = UserModel
    template_name = "users/user_edit_form.html"
    form_class = PublicProfileForm

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(EditProfile, self).dispatch(*args, **kwargs)

    def get_object(self, queryset=None):
        return self.request.user

    def get_context_data(self, **kwargs):
        context = super(EditProfile, self).get_context_data(**kwargs)
        context['settings_form'] = ProfileSettingsForm(instance=self.get_object())
        return context
edit_profile = EditProfile.as_view()


class EditProfileSettings(EditProfile):
    form_class = ProfileSettingsForm

    def get_context_data(self, **kwargs):
        context = super(EditProfileSettings, self).get_context_data(**kwargs)
        context['settings_form'] = self.form_class
        context['form'] = PublicProfileForm()
        return context

    def form_valid(self, form, *args, **kwargs):
        messages.success(self.request, "Your settings have been updated.")
        theme = form.cleaned_data.get('theme')
        if theme:
            self.request.COOKIES['theme'] = theme
        return super(EditProfile, self).form_valid(form, *args, **kwargs)
edit_settings = EditProfileSettings.as_view()


def view_profile(request, slug='', pk=''):
    """ Returns detail view for a single user """
    if pk:
        user = get_object_or_404(UserModel, id=pk)
    else:
        user = get_object_or_404(UserModel, username=slug)

    if request.is_ajax():
        location = user.city
        if user.state:
            location += ", {}".format(user.state)
        xhr_dict = {
            'name': user.display_name,
            'username': user.username,
            'joined': user.date_joined.strftime('%m/%d/%Y'),
            'location': location or '',
            'website': user.homepage or '',
            'profile_url': user.get_absolute_url(),
            'contact_url': reverse('contact_member', args=(user.username,)),
        }
        return HttpResponse(json.dumps(xhr_dict), mimetype='application/javascript')
    return render(request, "users/user_detail.html", {'user': user})



import datetime

from django.conf import settings
from django.db import models


RESTRICT_CONTENT_TO_SITE = getattr(settings, 'RESTRICT_CONTENT_TO_SITE', False)
now = datetime.datetime.now()


class VideoManager(models.Manager):
    """
    If RESTRICT_CONTENT_TO_SITE is True in settings,
    will limit video to current site.

    Usage is simply video.objects.all()
    """
    def get_queryset(self):
        videos = super(VideoManager, self).get_queryset()
        if RESTRICT_CONTENT_TO_SITE:
            videos.filter(sites__id__exact=settings.SITE_ID)
        return videos


class PublishedVideoManager(VideoManager):
    """
    Extends VideoManager to only return videos
    - That are published
    - and have a created date greater than or equal to now.

    Usage is gallery.published.all()
    """
    def get_queryset(self):
        videos = super(PublishedVideoManager, self).get_queryset()
        videos.filter(published=True, created__lte=now)
        return videos



import django_filters

from django.contrib.auth import get_user_model

UserModel = get_user_model()


class ProfileFilter(django_filters.FilterSet):
    display_name = django_filters.CharFilter(lookup_type='icontains')

    class Meta:
        model = UserModel
        fields = ['display_name']



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open('docs/requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='django-tango',
    version='0.9.1',
    author=u'Tim Baxter',
    author_email='mail.baxter@gmail.com',
    url='https://github.com/tBaxter/Tango',
    license='MIT',
    description='Faster, simpler Django content management.',
    long_description=open('README.md').read(),
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    install_requires=required,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
    ],
)



try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse

try:
    from urllib import urlopen
except ImportError:
    from urllib.request import urlopen #py3

from django.template.defaultfilters import slugify

from xmltramp2 import xmltramp

xml_media = xmltramp.Namespace('http://search.yahoo.com/mrss/')


def get_youtube_data(video):
    """
    Helper to extract video and thumbnail from youtube
    """
    video.source = 'youtube'
    if 'youtube.com/watch' in video.url:
        parsed = urlparse.urlsplit(video.url)
        query  = urlparse.parse_qs(parsed.query)
        try:
            video.key  = query.get('v')[0]
        except IndexError:
            video.key = None
    else:
        video.key = video.url.rsplit('/', 1)[1]
    video.embed_src = 'http://www.youtube.com/embed/'
    #http://gdata.youtube.com/feeds/api/videos/Agdvt9M3NJA
    api_url = 'http://gdata.youtube.com/feeds/api/videos/{}'.format(video.key)
    video_data = urlopen(api_url).read()
    xml = xmltramp.parse(video_data)

    video.title = unicode(xml.title)
    video.slug = slugify(video.title)
    video.summary = unicode(xml.content)
    video.thumb_url = xml[xml_media.group][xml_media.thumbnail:][1]('url')
    return video


def get_vimeo_data(video):
    """
    Helper to extract video and thumbnail from vimeo.
    """
    #http://vimeo.com/67325705 -- Tumbleweed Tango
    video.source = 'vimeo'
    video.key = video.url.rsplit('/', 1)[1]
    video.embed_src = 'http://player.vimeo.com/video/'

    api_url = 'http://vimeo.com/api/v2/video/{}.xml'.format(video.key)
    video_data = urlopen(api_url).read()
    xml = xmltramp.parse(video_data)
    video.title = unicode(xml.video.title)
    video.slug = slugify(video.title)
    video.summary = unicode(xml.video.description)
    video.thumb_url = unicode(xml.video.thumbnail_large)
    return video


def get_ustream_data(video):
    """
    Helper to extract video and thumbnail from ustream.
    """

    video.source = 'ustream'
    video.key = video.url.rsplit('/', 1)[1]
    video.embed_src = 'http://www.ustream.tv/embed/'
    if 'recorded' in video.url:
        video.embed_src += '/recorded/'
    video.title = "Ustream video"
    video.slug = 'ustream-{}'.format(video.key)
    return video



from django.conf import settings
from django.views.generic import ListView, DetailView

from video.models import Video, VideoGallery

paginate_by = getattr(settings, "PAGINATE_BY", 25)


class VideoList(ListView):
    """
    Returns a list of all videos.
    """
    template_name = 'video/video_list.html'
    paginate_by = paginate_by
    model = Video
video_list = VideoList.as_view()


class VideoDetail(DetailView):
    """
    Returns a video detail
    """
    template_name = 'video/video_detail.html'
    model = Video
video_detail = VideoDetail.as_view()


class VideoGalleryList(ListView):
    """
    Returns a list of all video Galleries.
    """
    template_name = 'video/gallery_list.html'
    paginate_by = paginate_by
    model = VideoGallery
video_gallery_list = VideoGalleryList.as_view()


class VideoGalleryDetail(DetailView):
    """
    Returns a gallery detail
    """
    template_name = 'video/video_list.html'
    model = VideoGallery
    context_object_name = 'gallery'

    def get_context_data(self, **kwargs):
        context = super(VideoGalleryDetail, self).get_context_data(**kwargs)
        context.update({
            'object_list' : self.get_object().video_collection.all(),
        })
        return context
video_gallery_detail = VideoGalleryDetail.as_view()



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin

from .models import Profile
from .forms import ProfileCreationForm


def deactivate(modeladmin, request, queryset):
    queryset.update(is_active=True)
deactivate.short_description = "Deactivate selected users"


class ProfileAdmin(UserAdmin):
    add_form = ProfileCreationForm

    date_hierarchy = 'date_joined'
    list_filter = ('is_staff', 'is_superuser', 'is_active', 'groups', 'date_joined')
    list_display = ('username', 'display_name', 'first_name', 'last_name', 'email', 'is_active', 'is_staff')
    search_fields = ('username', 'first_name', 'last_name', 'email', 'display_name')
    actions = [deactivate, ]


    def get_form(self, request, obj=None, **kwargs):
        """
        Use special form during user creation
        """
        defaults = {}
        if obj is None:
            defaults['form'] = self.add_form
        defaults.update(kwargs)
        return super(ProfileAdmin, self).get_form(request, obj, **defaults)

admin.site.register(Profile, ProfileAdmin)



from django import forms

from video.models import Video


class VideoForm(forms.ModelForm):
    class Meta:
        model = Video
        fields = ['title', 'summary', 'url', 'video_at_top', 'hide_info',]

    def __init__(self, *args, **kwargs):
        super(VideoForm, self).__init__(*args, **kwargs)
        self.fields['title'].help_text = "We'll attempt to fill this in from the video service."
        self.fields['title'].required = False
        self.fields['summary'].help_text = "We'll attempt to fill this in from the video service."
        self.fields['summary'].required = False



from django.template import Template, Context
from django.test import TestCase
from django.urls import reverse

from video.models import Video, VideoGallery
from .forms import VideoForm


class TestVideo(TestCase):

    def test_youtube_video(self):
        """
        Test that newly saved youtube videos
        are correctly capturing values from youtube.
        """
        url = 'https://www.youtube.com/watch?v=c-wXZ5-Yxuc'
        test_video = Video(url=url)
        test_video.save()
        self.assertIsInstance(test_video, Video)
        self.assertEqual(test_video.source, 'youtube')

        video_dict = test_video.__dict__
        self.assertIn('title', video_dict)
        self.assertIn('slug', video_dict)
        self.assertIn('summary', video_dict)
        self.assertIn('thumb_url', video_dict)

        response = self.client.get(reverse('video_detail', args=[test_video.slug, ]))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed('video_detail.html')
        self.assertIn('object', response.context)
        object = response.context['object']
        self.assertIsInstance(object, Video)
        self.assertTrue(hasattr(object, 'title'))
        self.assertTrue(hasattr(object, 'summary'))
        self.assertTrue(hasattr(object, 'embed_src'))

        # test template Tag:
        out = Template(
            "{% load video_tags %}"
            "{% show_video object %}"
        ).render(Context({'object': object}))
        self.assertIn(object.title, out)
        self.assertIn(object.embed_src, out)

    def test_vimeo_video(self):
        """
        Test that newly saved vimeo videos
        are correctly capturing values from youtube.
        """
        url = 'http://vimeo.com/67325705'
        test_video = Video(url=url)
        test_video.save()
        self.assertIsInstance(test_video, Video)
        self.assertEqual(test_video.source, 'vimeo')
        self.assertTrue('Tango' in test_video.title)

        video_dict = test_video.__dict__
        self.assertIn('title', video_dict)
        self.assertIn('slug', video_dict)
        self.assertIn('summary', video_dict)
        self.assertIn('thumb_url', video_dict)

        response = self.client.get(reverse('video_detail', args=[test_video.slug, ]))

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed('video_detail.html')
        self.assertIn('object', response.context)
        object = response.context['object']
        self.assertIsInstance(object, Video)

    def test_recent_video_tag(self):
        out = Template(
            "{% load video_tags %}"
            "{% get_video_list as video_list %}"
            "{% for video in video_list %}"
            "{{ video.title }},"
            "{% endfor %}"
        ).render(Context())
        self.assertEqual(out, '')

    def test_video_form(self):
        fields = VideoForm().fields
        self.assertFalse(fields['title'].required)
        self.assertFalse(fields['summary'].required)

    def test_video_gallery_detail(self):
        gallery = VideoGallery.objects.create(title= 'test gallery', slug='test-gallery')
        response = self.client.get(reverse('video_gallery_detail', args=[gallery.slug, ]))
        self.assertEqual(response.status_code, 200)
        self.assertIn('object', response.context)
        self.assertIn('gallery', response.context)
        self.assertTemplateUsed('video/video_list.html')

        #ensure we also attempted to pass a video list
        self.assertIn('object_list', response.context)



import os

from django.conf import settings
from django.contrib.auth.models import AbstractUser
from django.db import models
from django.urls import reverse
from django.utils.html import strip_tags

from tango_shared.models import set_img_path
from tango_shared.utils.sanetize import sanetize_text
from tango_shared.utils.maptools import get_geocode

THEME_CHOICES = [(theme, theme.capitalize()) for theme in getattr(settings, 'ALLOWABLE_THEMES', [])]


class Profile(AbstractUser):
    """
    Subclasses AbstractUser to provide site-specific user fields.
    """
    display_name = models.CharField(
        max_length=200,
        blank=True,
        null=True,
        help_text="""
        If you would prefer a different screen name, enter it here.
        Spaces are allowed. Note: your username will not change.
        """
    )
    street_address = models.CharField(
        max_length=255,
        blank=True,
        help_text="Will never be shown publicly on the site.")
    city = models.CharField(max_length=200, blank=True)
    state = models.CharField(max_length=200, blank=True)
    country = models.CharField(max_length=200, blank=True)
    zipcode = models.CharField(
        'ZIP/Postal Code',
        max_length=10,
        blank=True,
        help_text="Will never be shown publicly on the site."
    )
    interests = models.CharField(max_length=200, blank=True)
    occupation = models.CharField(max_length=200, blank=True)
    birthday = models.DateField(blank=True, null=True)
    homepage = models.URLField('Your web site', blank=True)
    bio = models.TextField(help_text='Tell us a bit about yourself.', blank=True, null=True)
    bio_formatted = models.TextField(blank=True, editable=False)
    signature = models.CharField(
        max_length=255,
        blank=True,
        help_text="""
            You can have a short signature line on the posts and comments.
            Members who choose to view signatures can see it. HTML is not allowed.
            """
    )
    geocode = models.CharField(max_length=200, null=True, blank=True)
    avatar = models.ImageField(blank=True, null=True, upload_to=set_img_path)
    if 'fretboard' in settings.INSTALLED_APPS:
        post_count = models.IntegerField(default="0", editable=False)

    #preferences
    display_on_map = models.BooleanField(default=True)
    open_links = models.BooleanField(
        "Open links in new window",
        default=False,
        help_text="Check if you would like links to automatically open in a new window."
    )
    show_signatures = models.BooleanField(
        default=False,
        help_text="Check if you would like to see signatures attached to forum posts."
    )
    theme = models.CharField(max_length=100, blank=True, choices=THEME_CHOICES)

    class Meta:
        ordering = ('display_name',)

    def __unicode__(self):
        return self.username

    def save(self, *args, **kwargs):
        needs_geocode = False
        if self.id is None:  # For new user, only set a few things:
            self.display_name = self.get_display_name()
            needs_geocode = True
        else:
            old_self = self.__class__.objects.get(id = self.id)
            if old_self.city != self.city or old_self.state != self.state or self.geocode is None:
                needs_geocode = True
            if old_self.display_name != self.display_name or old_self.first_name != self.first_name or old_self.last_name != self.last_name:
                self.display_name = self.get_display_name()  # check if display name has changed
            if old_self.avatar and old_self.avatar != self.avatar:
                os.remove(old_self.avatar.path)
        if self.city and self.state and needs_geocode:
            geocode = get_geocode(self.city, self.state, street_address=self.street_address, zipcode=self.zip)
            if geocode and geocode != '620':
                self.geocode = ', '.join(geocode)
        if self.signature:
            self.signature = strip_tags(self.signature)
        if self.bio:
            self.bio_formatted = sanetize_text(self.bio)

        super(Profile, self).save(*args, **kwargs)

    # fix this for current URL -- use permalink
    def get_absolute_url(self):
        return reverse('view_profile', args=[self.username])

    def get_display_name(self):
        """
        Determined display screen name based on the first of
        display_name, full name, or username.
        """
        if self.display_name:
            return self.display_name
        elif self.first_name and self.last_name:
            return '{0} {1}'.format(self.first_name, self.last_name)
        else:
            return self.username
